/*
Copyright Ideaconsult Ltd. (C) 2005-2007 

Contact: nina@acad.bg

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.
All we ask is that proper credit is given for our work, which includes
- but is not limited to - adding the above copyright notice to the beginning
of your source code files, and to any copyright notice that you may distribute
with programs based on this work.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

*//**
 * Created on 2005-5-2

 * @author Nina Jeliazkova nina@acad.bg
 *
 * Project : toxtree
 * Package : toxTree.tree.rules
 * Filename: RuleHeteroaromatic.java
 */
package toxTree.tree.cramer;

import org.openscience.cdk.interfaces.IAtomContainer;

import toxTree.exceptions.DecisionMethodException;
import toxTree.query.MolFlags;
import toxTree.tree.rules.smarts.RuleSMARTSSubstructureAmbit;

/**
 * Rule 12 of the Cramer scheme (see {@link toxTree.tree.cramer.CramerRules})
 * @author Nina Jeliazkova <br>
 * @version 0.1, 2005-5-2
 */
public class RuleHeteroaromatic extends RuleSMARTSSubstructureAmbit {

    private static final long serialVersionUID = -4930724854762812531L;
    /**
	 * Constructor
	 * 
	 */
	public RuleHeteroaromatic() {
		super();
		id = "12";
		try {addSubstructure("heteroaromatic","a:[a;!c]:a"); } catch (Exception x) {x.printStackTrace();}
		title = "Heteroaromatic";
		explanation.append("<html>This question separates the aromatic heterocyclics for the purpose of considering whether they are polynuclear(Q14) or unsubstituted (Q13).</html>");
		//both examples are heteroaromatics!
		examples[0] = "S1C=CN=C1";
		examples[1] = "O1C=CC=C1";
		editable = false;
		
	}
	/**
	 * {@link toxTree.core.IDecisionRule#verifyRule(IAtomContainer)}
	 */
	public boolean verifyRule(IAtomContainer  mol) throws DecisionMethodException {
		logger.finer(toString());
	    //should be set via MolAnalyser
	    MolFlags mf = (MolFlags) mol.getProperty(MolFlags.MOLFLAGS);
	    if (mf ==null) throw new DecisionMethodException(ERR_STRUCTURENOTPREPROCESSED);
	    if (mf.isHeteroaromatic()) {
	    	logger.finer("Heteroaromatic\t"+MSG_YES);
	    	return true;
	    } else {
	    	logger.finer("Heteroaromatic\t"+MSG_NO);
	    	return false;
	    }
	    
	}
	/* (non-Javadoc)
     * @see toxTree.tree.AbstractRule#isImplemented()
     */
    @Override
	public boolean isImplemented() {
        return true;
    }	
}
